from __future__ import print_function
import os
import pandas as pd
import numpy as np
from datetime import date, datetime
import pickle
import ariskparametri
from collections import defaultdict
from jinja2 import Environment, FileSystemLoader
from treeinterpreter import treeinterpreter as ti
from ariskutil import myprint, s2fwithsep, stringFromJson, try_div, f2swithsep
import random
import copy
from collections import OrderedDict
import csv
import time
import matplotlib.pyplot as plt


def analize_company_explain(loaded_model, y_pred_r, mycompanyData):
    y_pred_r = np.round(y_pred_r, 1)

    optimised_random_forest = loaded_model.best_estimator_.steps[-1]
    prediction, bias, contributions = ti.predict(optimised_random_forest[1][1], mycompanyData)

    localRes = list()

    y_pred_rColors = list()
    for i in range(len(y_pred_r)):
        if y_pred_r[i] > ariskparametri.YearlyYellow:
            if y_pred_r[i] > ariskparametri.YearlyRed:
                y_pred_rColors.append(ariskparametri.ReportBarRosso)
            else:
                y_pred_rColors.append(ariskparametri.ReportBarGiallo)
        else:
            y_pred_rColors.append(ariskparametri.ReportBarVerde)

    for i in range(len(contributions)):
        res0, res1 = map(list, zip(*contributions[i]))
        localRes.append(res1)

    return localRes


def analyze_company(mycompanyInfo, mycompanyData, indFamiliarita, sev_array):
    y_pred = list()
    y_pred_r = list()

    mycompanyData = mycompanyData.values.reshape(1, -1)

    myprint(os.getcwd())
    loaded_model = pickle.load(open(ariskparametri.fileTrain1Y, 'rb'))
    y_pred_r.extend(loaded_model.predict_proba(mycompanyData)[:, 1])
    loaded_model = pickle.load(open(ariskparametri.fileTrain2Y, 'rb'))
    y_pred_r.extend(loaded_model.predict_proba(mycompanyData)[:, 1])
    loaded_model = pickle.load(open(ariskparametri.fileTrain3Y, 'rb'))
    y_pred_r.extend(loaded_model.predict_proba(mycompanyData)[:, 1])
    y_pred_r = np.round(y_pred_r, 1)
    optimised_random_forest = loaded_model.steps[-1]
    prediction, bias, contributions = ti.predict(optimised_random_forest[1][1], mycompanyData)

    res = list()

    y_pred_rColors = list()
    for i in range(len(y_pred_r)):
        if y_pred_r[i] > ariskparametri.YearlyYellow:
            if y_pred_r[i] > ariskparametri.YearlyRed:
                y_pred_rColors.append(ariskparametri.ReportBarRosso)
            else:
                y_pred_rColors.append(ariskparametri.ReportBarGiallo)
        else:
            y_pred_rColors.append(ariskparametri.ReportBarVerde)

    allValue = 0.0
    for i in range(3):
        allValue = allValue + y_pred_r[i] * ariskparametri.mediaPesataAnni[i]

    if allValue > ariskparametri.OverallYellow:
        if allValue > ariskparametri.OverallRed:
            allValueColor = ariskparametri.ReportBarRosso
            allTableColor = ariskparametri.ReportSfondoRosso
        else:
            allValueColor = ariskparametri.ReportBarGiallo
            allTableColor = ariskparametri.ReportSfondoGiallo
    else:
        allValueColor = ariskparametri.ReportBarVerde
        allTableColor = ariskparametri.ReportSfondoVerde

    for i in range(len(contributions)):
        res0, res1 = map(list, zip(*contributions[i]))
        res.append(res1)

    redDetResNames = list()
    redDetResValues = list()
    redDetResIdx = list()
    flattenData = mycompanyData.flatten()
    for i in np.argsort(np.negative(res)).flatten():
        if res[0][i] >= 0.001:
            redDetResNames.append(ariskparametri.indices[i])
            redDetResValues.append(flattenData[i])
            redDetResIdx.append(i)

    AI4BCPredInput = mycompanyData
    delta, AI4BCPredValue, AI4BCNames = analyze_companyBC(AI4BCPredInput, sev_array)

    if AI4BCPredValue > ariskparametri.AI4BCYellow:
        if AI4BCPredValue > ariskparametri.AI4BCRed:
            AI4BCColor = ariskparametri.ReportSfondoRosso
        else:
            AI4BCColor = ariskparametri.ReportSfondoGiallo
    else:
        AI4BCColor = ariskparametri.ReportSfondoVerde

    AI4RiskPredInput = mycompanyData
    delta, AI4RiskPredValue, AI4RiskNames = analyze_companyGov(AI4RiskPredInput, redDetResIdx, indFamiliarita)

    AI4RisksPredValue = 0.0

    loaded_model = pickle.load(open(ariskparametri.fileTrain1Y, 'r'))
    AI4RisksPredValue = AI4RisksPredValue + (loaded_model.predict_proba(AI4RiskPredInput)[:, 1])
    loaded_model = pickle.load(open(ariskparametri.fileTrain2Y, 'r'))
    AI4RisksPredValue = AI4RisksPredValue + (loaded_model.predict_proba(AI4RiskPredInput)[:, 1])
    loaded_model = pickle.load(open(ariskparametri.fileTrain3Y, 'r'))
    AI4RisksPredValue = AI4RisksPredValue + (loaded_model.predict_proba(AI4RiskPredInput)[:, 1])
    AI4RisksPredValue = AI4RisksPredValue / 3

    if AI4RisksPredValue > ariskparametri.AI4RiskYellow:
        if AI4RisksPredValue > ariskparametri.AI4RiskRed:
            AI4RiskColor = ariskparametri.ReportSfondoRosso
        else:
            AI4RiskColor = ariskparametri.ReportSfondoGiallo
    else:
        AI4RiskColor = ariskparametri.ReportSfondoVerde

    env = Environment(loader=FileSystemLoader(os.path.join(os.path.dirname(__file__), 'template-report')))
    template = env.get_template('template-short.html')
    report = template.render(Azienda=mycompanyInfo['ATT1'].values[0],
                             companyAnagr=['Azienda', 'Partita IVA', 'Codice ATECO 2007'],
                             companyAnagrValue=[mycompanyInfo['ATT1'].values[0], mycompanyInfo['ATT3'].values[0],
                                                mycompanyInfo['ATT6'].values[0]],
                             companyData=ariskparametri.indices,
                             companyDataValue=flattenData,
                             Values=y_pred_r,
                             ScoreParams=redDetResNames,
                             ScoreParamsValue=redDetResValues,
                             ScoreParamsText=redDetResNames,
                             OverallValue=allValue,
                             OverallValueColor=allValueColor,
                             ValuesColor=y_pred_rColors,
                             AI4BCValue=AI4BCPredValue,
                             AI4RisksValue=AI4RisksPredValue,
                             reportCreated=date.today(),
                             EQFStart=mycompanyInfo['EQFStart'].values[0],
                             EQFEnd=mycompanyInfo['EQFEnd'].values[0],
                             AQStart=mycompanyInfo['AQStart'].values[0],
                             AQEnd=mycompanyInfo['AQStart'].values[0],
                             AI4REDFlagsLevelColor=allTableColor,
                             AI4BCLevelColor=AI4BCColor,
                             AI4RiskLevelColor=AI4RiskColor,
                             AI4BCScoreParams=AI4BCNames,
                             AI4RiskScoreParams=AI4RiskNames
                             )
    return report


def analyze_company_rf(mycompanyData1, ratio=1.0):

    results = {}
    mycompanyData = copy.deepcopy(mycompanyData1)
    y_pred = list()
    y_pred_r = list()
    mycompanyData = mycompanyData.values.reshape(1, -1)
    myprint(os.getcwd())
    loaded_model = pickle.load(open(ariskparametri.fileTrain1Y, 'rb'))
    y_pred_r.extend(loaded_model.predict_proba(mycompanyData)[:, 1])

    optimised_random_forest = loaded_model.steps[-1]
    prediction, bias, contributions = ti.predict(optimised_random_forest[1][1], mycompanyData)

    loaded_model = pickle.load(open(ariskparametri.fileTrain2Y, 'rb'))
    y_pred_r.extend(loaded_model.predict_proba(mycompanyData)[:, 1])

    loaded_model = pickle.load(open(ariskparametri.fileTrain3Y, 'rb'))
    y_pred_r.extend(loaded_model.predict_proba(mycompanyData)[:, 1])

    optimised_random_forest = loaded_model.steps[-1]
    prediction, bias, contributions = ti.predict(optimised_random_forest[1][1], mycompanyData)

    res = list()

    y_pred_rColors = list()
    for i in range(len(y_pred_r)):
        if y_pred_r[i] > ariskparametri.YearlyYellow:
            if y_pred_r[i] > ariskparametri.YearlyRed:
                y_pred_rColors.append(ariskparametri.ReportBarRosso)
            else:
                y_pred_rColors.append(ariskparametri.ReportBarGiallo)
        else:
            y_pred_rColors.append(ariskparametri.ReportBarVerde)

    allValue = 0.0
    for i in range(3):
        allValue = allValue + y_pred_r[i] * ariskparametri.mediaPesataAnni[i]

    results['ai4rfmeanres'] = allValue
    if allValue > ariskparametri.OverallYellow:
        if allValue > ariskparametri.OverallRed:
            results['ai4rfmeanresSeverity'] = ariskparametri.ReportSfondoRosso
        else:
            results['ai4rfmeanresSeverity'] = ariskparametri.ReportSfondoGiallo
    else:
        results['ai4rfmeanresSeverity'] = ariskparametri.ReportSfondoVerde

    for i in range(len(contributions)):
        res0, res1 = map(list, zip(*contributions[i]))
        res.append(res1)
    myprint(contributions)
    myprint(res)
    myprint(np.argsort(np.negative(res)))
    redDetResNames = list()
    redDetResValues = list()
    redDetResIdx = list()
    resDetResContrType= list()
    flattenData = mycompanyData.flatten()
    for i in np.argsort(np.negative(res)).flatten():
        myprint(i)
        if res[0][i] >= 0.001:
            myprint(str(res[0][i]) + " " + ariskparametri.indices[i])
            redDetResNames.append(ariskparametri.indices[i])
            redDetResValues.append(flattenData[i])
            redDetResIdx.append(i)

    for i in range(len(res[0])):
        if res[0][i] > 0:
            resDetResContrType.append(1.0)
        elif res[0][i] < 0:
            resDetResContrType.append(-1.0)
        else:
            resDetResContrType.append(0.0)
    return results, redDetResNames, redDetResValues, resDetResContrType


def get_crisis(test, trained_filename, level):

    loaded_model = pickle.load(open(trained_filename, 'rb'))
    optimised_random_forest = loaded_model.steps[-1]
    prediction, bias, contributions = ti.predict(optimised_random_forest[1][1], test)
    y_predict = (loaded_model.predict_proba(test)[:, 1] >= level)
    y_predict_r = loaded_model.predict_proba(test)[:, 1]
    y_predict_r = np.round(y_predict_r, 2)

    return y_predict, y_predict_r, contributions


def get_detailed_report(X, y, trainedFilename, level):

    loaded_model = pickle.load(open(trainedFilename, 'rb'))
    y_pred = (loaded_model.predict_proba(X)[:, 1] >= level)
    y_pred_r = loaded_model.predict_proba(X)[:, 1]

    y_pred_r = np.round(y_pred_r, 1)

    optimised_random_forest = loaded_model.best_estimator_.steps[-1]
    prediction, bias, contributions = ti.predict(optimised_random_forest[1][1], X)

    indices = [
        'Revenues', #'Ricavi delle vendite',
        'EBITDA Margin', #'Ricavi delle vendite',
        'EBITDA', #'EBITDA migl EUR'
        'Quick ratio', #'Indice di liquidità',
        'Current ratio', #'Indice corrente',
        'Long-Term Liability Ratio', #'Indice di indebitam. a lungo',
        'Coverage Of Fixed Assets', #'Indice di copertura delle immob. (patrimoniale)',
        'Amortisation Rate', #'Grado di ammortamento',
        'Debt On Sales', #'Debiti su fatt. (%)',
        'Interest Coverage Ratio', #'Grado di copertura degli interessi passivi',
        'Average Stock Turnover', #'Giac. media delle scorte (gg)',
        'Stock Coverage Days', #'Giorni copertura scorte (gg)',
        'Return On Investments  (ROI)', #'Redditività di tutto il capitale investito (ROI) (%)',
        'Cash Flow', #'Flusso di cassa di gestione',
        'Interest On Sales', #'Oneri finanz. su fatt. (%)'
        'Net Working Capital', #Capitale Circolante Netto
        'Net Financial Position', #Posizione Finanziaria netta
        'Net Short-Term Assets', #Margine di Tesoreria
        'Shareholder Funds-Fixed Assets', #Margine di Struttura
        'Return on Asset (ROA)', #ROA
        'Return on Sales (ROS)', #ROS
        'Return on Equity (ROE)', #ROE
        'Type of Financial Statement' #NEW MPAI a scelta tra Abbreviated, Micro, Detailed
    ]

    for i in range(5):
        print(bias[0])

        print("Feature contributions:")
        res0, res1 = map(list, zip(*contributions[i]))
        for c, feature in sorted(zip(res1, indices), key=lambda x: -abs(x[0])):
            print(feature, abs(round(c, 2)))
        print("-" * 20)

    res = list()
    for i in range(len(contributions)):
        res0, res1 = map(list, zip(*contributions[i]))
        res.append(res1)

    detRes = pd.DataFrame(np.abs(res), columns=indices)

    return detRes


def JsonDataEmpty(jSonDataToCheck):
    if (jSonDataToCheck is None) or (not str(jSonDataToCheck).strip()):
        return False
    return True


def checkSurveyData(jSonData):
    checkResult = True
    if jSonData['InData']['AQLOCKED'] is not None:
        if 'INDFINANZIARI' in jSonData:
            indFinanziari = jSonData['InData']['AQLOCKED']['input'][2]['fixedRows']
            checkResult = checkResult and JsonDataEmpty(indFinanziari[8]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[9]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[10]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[5]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[11]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[12]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[13]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[14]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[15]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[16]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[17]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[18]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[19]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[20]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[21]['values'][0]['value'])
        else:
            checkResult = \
                checkResult and \
                JsonDataEmpty(jSonData['InData']['AQLOCKED'][0]['input'][16]['fixedRows'][0]['values'][0]['value'])
    return checkResult


def checkSurveyData1(jSonData):
    checkResult = True
    if jSonData['InData']['AQLOCKED'] is not None:
        if 'INDFINANZIARI' in jSonData:
            indFinanziari = jSonData['InData']['AQLOCKED'][0]['input'][2]['fixedRows']
            checkResult = checkResult and JsonDataEmpty(indFinanziari[8]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[9]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[10]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[5]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[11]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[12]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[13]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[14]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[15]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[16]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[17]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[18]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[19]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[20]['values'][0]['value'])
            checkResult = checkResult and JsonDataEmpty(indFinanziari[21]['values'][0]['value'])
        else:
            checkResult = \
                checkResult and \
                JsonDataEmpty(jSonData['InData']['AQLOCKED'][0]['input'][17]['fixedRows'][0]['values'][0]['value'])

    return checkResult


def analyze_companyBC_batch(predinput1, rf):
    gamma = [
        [-0.05, -0.05, -0.05, -0.05, 0.05, -0.05, 0.05, -0.05, 0.05],
        [-0.1, -0.1, -0.1, -0.1, 0.1, -0.1, 0.1, -0.1, 0.1],
        [-0.15, -0.15, -0.15, -0.15, 0.15, -0.15, 0.15, -0.15, 0.15],
        [-.2, -.2, -.2, -.2, .2, -.2, .2, -.2, .2],
        [-.5, -.5, -.5, -.5, .5, -.5, .5, -.5, .5],
        [-.8, -.8, -.8, -.8, .8, -.8, .8, -.8, .8]
    ]
    severity_array = [
        [1,0,0,0,0,0,0,0],
        [0,1,0,0,0,0,0,0],
        [0,0,1,0,0,0,0,0],
        [0,0,0,1,0,0,0,0],
        [0,0,0,0,1,0,0,0],
        [0,0,0,0,0,1,0,0],
        [0,0,0,0,0,0,1,0],
        [0,0,0,0,0,0,0,1]
    ]
    with open('./BC_RF.csv', 'w') as wrtr:
        writer = csv.writer(wrtr)
        writer.writerow(
            ['gamma', 'SOCI,FIN,STR,CYB,TUTELA,AMB,COMPL,ALTRI',
             'ricavi,EBIDTA,EBIDTA/Sales,liqui,indebitam,copertura,totale,ROI,oneri', 'BC', 'RF'])
    for gm in range(len(gamma)):
        for row in range(len(severity_array)):
            PredInput = copy.deepcopy(predinput1)

            results = {}

            if PredInput.loc[0][1] < 0:
                gamma[gm][1] = -gamma[gm][1]
            if PredInput.loc[0][2] < 0:
                gamma[gm][2] = -gamma[gm][2]
            if PredInput.loc[0][12] < 0:
                gamma[gm][7] = -gamma[gm][7]

            temp = list()
            for i in range(len(severity_array)):
                temp.append([])
                for j in range(len(ariskparametri.alpha2)):
                    temp[i].append((severity_array[row][i] *
                                   (gamma[gm][j] * (ariskparametri.alpha2[j] + ariskparametri.beta2[i][j]))))

            temp = np.array(temp)
            arr = temp.sum(axis=0)
            PredInput.loc[0][0] = PredInput.loc[0][0] * (1 + arr[0])
            PredInput.loc[0][1] = PredInput.loc[0][1] * (1 + arr[1])
            PredInput.loc[0][2] = PredInput.loc[0][2] * (1 + arr[2])
            PredInput.loc[0][3] = PredInput.loc[0][3] * (1 + arr[3])
            PredInput.loc[0][5] = PredInput.loc[0][5] * (1 + arr[4])
            PredInput.loc[0][6] = PredInput.loc[0][6] * (1 + arr[5])
            PredInput.loc[0][8] = PredInput.loc[0][8] * (1 + arr[6]*.5)  
            if arr[7] <= -1:
                PredInput.loc[0][12] = PredInput.loc[0][12] * 0.9
            else:
                PredInput.loc[0][12] = PredInput.loc[0][12] * (1 + arr[7])
            PredInput.loc[0][14] = PredInput.loc[0][14] * (1 + arr[8]*.5)  # * 0.5 because gamma is 0.05 for it
            y_pred_r = list()

            loaded_model = pickle.load(open(ariskparametri.fileTrain1Y, 'rb'))
            y_pred_r.extend(loaded_model.predict_proba(PredInput)[:, 1])
            myprint("anno 1" + " " + str(loaded_model.predict_proba(PredInput)[:, 1]))
            loaded_model = pickle.load(open(ariskparametri.fileTrain2Y, 'rb'))
            y_pred_r.extend(loaded_model.predict_proba(PredInput)[:, 1])
            myprint("anno 2" + " " + str(loaded_model.predict_proba(PredInput)[:, 1]))
            loaded_model = pickle.load(open(ariskparametri.fileTrain3Y, 'rb'))
            y_pred_r.extend(loaded_model.predict_proba(PredInput)[:, 1])
            myprint("anno 3" + " " + str(loaded_model.predict_proba(PredInput)[:, 1]))
            predValues = 0.0
            for i in range(3):
                predValues = predValues + y_pred_r[i] * ariskparametri.mediaPesataAnniBC[i]
            
            results['ai4rfmeanres'] = predValues
            if predValues > ariskparametri.OverallYellow:
                if predValues > ariskparametri.OverallRed:
                    results['ai4rfmeanresSeverity'] = ariskparametri.ReportSfondoRosso
                else:
                    results['ai4rfmeanresSeverity'] = ariskparametri.ReportSfondoGiallo
            else:
                results['ai4rfmeanresSeverity'] = ariskparametri.ReportSfondoVerde

            with open('./BC_RF.csv', 'a') as wrtr:
                writer = csv.writer(wrtr)
                writer.writerow([gamma[gm][0], severity_array[row], arr, predValues, rf])


def analyze_companyBC(PredInput1, severity_array1):
    PredInput = copy.deepcopy(PredInput1)
    severity_array = copy.deepcopy(severity_array1)
    gamma = copy.deepcopy(ariskparametri.gamma8)
    severity_array.pop('RISKMAPGDPR')
    results = {}

    if PredInput.loc[0][1] < 0:
        gamma[1] = -gamma[1]
    if PredInput.loc[0][2] < 0:
        gamma[2] = -gamma[2]
    if PredInput.loc[0][12] < 0:
        gamma[7] = -gamma[7]

    temp = list()
    for i in range(len(severity_array)):
        temp.append([])
        for j in range(len(ariskparametri.alpha2)):
            temp[i].append(list(severity_array.values())[i] *
                           (gamma[j] * (ariskparametri.alpha2[j] + ariskparametri.beta3[i][j])))

    temp = np.array(temp)
    arr = temp.mean(axis=0)
    for i in range(len(arr)):
        if arr[i] > 0.001:
            arr[i] = arr[i] + 0.3
        if arr[i] < -0.001:
            arr[i] = arr[i] - 0.3

    y_pred_r = list()

    loaded_model = pickle.load(open(ariskparametri.fileTrain1Y, 'rb'))
    y_pred_r.extend(loaded_model.predict_proba(PredInput)[:, 1])
    myprint("anno 1" + " " + str(loaded_model.predict_proba(PredInput)[:, 1]))
    loaded_model = pickle.load(open(ariskparametri.fileTrain2Y, 'rb'))
    y_pred_r.extend(loaded_model.predict_proba(PredInput)[:, 1])
    myprint("anno 2" + " " + str(loaded_model.predict_proba(PredInput)[:, 1]))
    loaded_model = pickle.load(open(ariskparametri.fileTrain3Y, 'rb'))
    y_pred_r.extend(loaded_model.predict_proba(PredInput)[:, 1])
    myprint("anno 3" + " " + str(loaded_model.predict_proba(PredInput)[:, 1]))
    predValues = 0.0
    for i in range(3):
        predValues = predValues + y_pred_r[i] * ariskparametri.mediaPesataAnniBC[i]
    results['ai4rfmeanres'] = predValues
    if predValues > ariskparametri.OverallYellow:
        if predValues > ariskparametri.OverallRed:
            results['ai4rfmeanresSeverity'] = ariskparametri.ReportSfondoRosso
        else:
            results['ai4rfmeanresSeverity'] = ariskparametri.ReportSfondoGiallo
    else:
        results['ai4rfmeanresSeverity'] = ariskparametri.ReportSfondoVerde

    return results



def analyze_companyGov(PredInput1):


    PredInput = copy.deepcopy(PredInput1)
    results = {}

    loaded_model = pickle.load(open(ariskparametri.fileTrainGov, 'rb'))
    print(loaded_model.predict(PredInput))
    predValues = loaded_model.predict(PredInput)
    results['ai4rfmeanres'] = int(predValues[0]) / 100
    if predValues > ariskparametri.OverallYellow:
        if predValues > ariskparametri.OverallRed:
            results['ai4rfmeanresSeverity'] = ariskparametri.ReportSfondoRosso
        else:
            results['ai4rfmeanresSeverity'] = ariskparametri.ReportSfondoGiallo
    else:
        results['ai4rfmeanresSeverity'] = ariskparametri.ReportSfondoVerde

    return results


def computeKPIs(AQSurvey, ratio=1.0):
    BILTYPE = AQSurvey[2]['value']

    my_kpi = {}
    my_kpi['Revenues'] = s2fwithsep(AQSurvey[17]['fixedRows'][0]['values'][0]['value'])
    my_kpi['totaleDebitiEntre12Mesi'] = s2fwithsep(AQSurvey[14]['fixedRows'][28]['values'][0]['value'])
    if BILTYPE == '3':
        my_kpi['Net Working Capital'] = 0
        my_kpi['Net Working Capital'] = \
            my_kpi['Net Working Capital'] \
            + s2fwithsep(AQSurvey[8]['fixedRows'][6]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][12]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][5]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[4]['fixedRows'][2]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][8]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][10]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][12]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][14]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][16]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][18]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][28]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[9]['fixedRows'][2]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][0]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][2]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][4]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][8]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][10]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][14]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][16]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][18]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][20]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][22]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][24]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][26]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[15]['fixedRows'][2]['values'][0]['value'])
    else:
        my_kpi['Net Working Capital'] = ariskparametri.NullValue
    my_kpi['Net Working CapitalAIDA'] = +s2fwithsep(AQSurvey[8]['fixedRows'][33]['values'][0]['value']) \
                                            - my_kpi['totaleDebitiEntre12Mesi']

    my_kpi['EBITDA'] = \
        - s2fwithsep(AQSurvey[18]['fixedRows'][0]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][1]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][2]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][3]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][4]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][8]['values'][0]['value']) \
        + s2fwithsep(AQSurvey[17]['fixedRows'][0]['values'][0]['value']) \
        + s2fwithsep(AQSurvey[17]['fixedRows'][3]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][14]['values'][0]['value'])

    my_kpi['EBITDAAIDA'] = round(s2fwithsep(AQSurvey[18]['fixedRows'][19]['values'][0]['value'])
                                 + s2fwithsep(AQSurvey[18]['fixedRows'][12]['values'][0]['value'])
                                 + s2fwithsep(AQSurvey[18]['fixedRows'][13]['values'][0]['value']), 2)


    my_kpi['totaleRimanenze'] = s2fwithsep(AQSurvey[8]['fixedRows'][5]['values'][0]['value'])

    tmp_value = 0.0
    tmp_value = \
        tmp_value + s2fwithsep(AQSurvey[8]['fixedRows'][20]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[8]['fixedRows'][21]['values'][0]['value']) \
        + s2fwithsep(AQSurvey[8]['fixedRows'][32]['values'][0]['value']) \
        + s2fwithsep(AQSurvey[9]['fixedRows'][2]['values'][0]['value']) \
        + s2fwithsep(AQSurvey[8]['fixedRows'][5]['values'][0]['value'])

    tmp_value2 = 0.0
    tmp_value2 = \
        tmp_value2 \
        + s2fwithsep(AQSurvey[14]['fixedRows'][28]['values'][0]['value']) \
        + s2fwithsep(AQSurvey[15]['fixedRows'][2]['values'][0]['value'])
    my_kpi['Current Ratio'] = try_div(tmp_value, tmp_value2)
    tmp_value = +s2fwithsep(AQSurvey[8]['fixedRows'][33]['values'][0]['value'])
    tmp_value2 = + s2fwithsep(AQSurvey[14]['fixedRows'][28]['values'][0]['value'])
    my_kpi['Current RatioAIDA'] = round(try_div(tmp_value, tmp_value2), 2)

    my_kpi['Quick Ratio'] = try_div((tmp_value - my_kpi['totaleRimanenze']), tmp_value2)

    tmp_value = +s2fwithsep(AQSurvey[8]['fixedRows'][33]['values'][0]['value']) - my_kpi['totaleRimanenze']
    tmp_value2 = + s2fwithsep(AQSurvey[14]['fixedRows'][28]['values'][0]['value'])
    my_kpi['Quick RatioAIDA'] = round(try_div(tmp_value, tmp_value2), 2)
    my_kpi['Net Short-Term Assets'] = tmp_value - tmp_value2
    if BILTYPE == '3':

        my_kpi['Net Short-Term AssetsAIDA'] = +(+s2fwithsep(AQSurvey[8]['fixedRows'][6]['values'][0]['value'])
                                           - s2fwithsep(AQSurvey[8]['fixedRows'][7]['values'][0]['value'])) \
                                         + (+s2fwithsep(AQSurvey[8]['fixedRows'][18]['values'][0]['value'])
                                            - s2fwithsep(AQSurvey[8]['fixedRows'][19]['values'][0]['value'])) \
                                         + s2fwithsep(AQSurvey[8]['fixedRows'][32]['values'][0]['value']) \
                                         - my_kpi['totaleDebitiEntre12Mesi']
    else:
        my_kpi['Net Short-Term AssetsAIDA'] = ariskparametri.NullValue

    my_kpi['totaleDebiti'] = s2fwithsep(AQSurvey[14]['fixedRows'][30]['values'][0]['value'])

    tmp_value = s2fwithsep(AQSurvey[14]['fixedRows'][6]['values'][0]['value'])

    my_kpi['patrimonioNetto'] = s2fwithsep(AQSurvey[11]['fixedRows'][9]['values'][0]['value'])

    if BILTYPE == '3':
        my_kpi['levaFinanziaria'] = try_div(+tmp_value, my_kpi['patrimonioNetto'])
    else:
        my_kpi['levaFinanziaria'] = ariskparametri.NullValue

    my_kpi['gradoIndebitamento'] = try_div(my_kpi['totaleDebiti'], my_kpi['patrimonioNetto'])

    my_kpi['totaleDebitiOltre12Mesi'] = s2fwithsep(AQSurvey[14]['fixedRows'][29]['values'][0]['value'])

    my_kpi['Long-Term Liability Ratio'] = try_div(my_kpi['totaleDebitiOltre12Mesi'], my_kpi['patrimonioNetto'])
    tmp_value = my_kpi['totaleDebitiEntre12Mesi'] + my_kpi['totaleDebitiOltre12Mesi']
    my_kpi['Long-Term Liability RatioAIDA'] = round(try_div(my_kpi['totaleDebitiOltre12Mesi'], tmp_value), 2)

    my_kpi['attivoFisso'] = s2fwithsep(AQSurvey[7]['fixedRows'][11]['values'][0]['value'])

    my_kpi['Coverage Of Fixed Assets'] = try_div(my_kpi['patrimonioNetto'], my_kpi['attivoFisso'])
    tmp_value = s2fwithsep(AQSurvey[6]['fixedRows'][5]['values'][0]['value'])
    my_kpi['Coverage Of Fixed AssetsAIDA'] = round(try_div(tmp_value, my_kpi['patrimonioNetto']), 2)

    my_kpi['Average Stock Turnover'] = try_div(my_kpi['totaleRimanenze'], (my_kpi['Revenues'] * ratio)) * 365

    if BILTYPE == '3':

        tmp_value = + s2fwithsep(AQSurvey[8]['fixedRows'][5]['values'][0]['value']) \
                    - s2fwithsep(AQSurvey[8]['fixedRows'][4]['values'][0]['value'])
        tmp_value2 = + my_kpi['Revenues'] + s2fwithsep(AQSurvey[17]['fixedRows'][5]['values'][0]['value'])
        my_kpi['Average Stock TurnoverAIDA'] = round(try_div(tmp_value, tmp_value2) * 365, 2)
    else:
        my_kpi['Average Stock TurnoverAIDA'] = ariskparametri.NullValue

    if BILTYPE == '3':
        tmp_value = + s2fwithsep(AQSurvey[17]['fixedRows'][1]['values'][0]['value']) \
                    - s2fwithsep(AQSurvey[18]['fixedRows'][0]['values'][0]['value']) \
                    - s2fwithsep(AQSurvey[18]['fixedRows'][14]['values'][0]['value'])

        tmp_value = tmp_value * ratio
        my_kpi['Stock Coverage Days'] = try_div(my_kpi['totaleRimanenze'], -tmp_value) * 365

    else:
        my_kpi['Stock Coverage Days'] = ariskparametri.NullValue

    if BILTYPE == '3':
        tmp_value = + s2fwithsep(AQSurvey[8]['fixedRows'][5]['values'][0]['value']) \
                    - s2fwithsep(AQSurvey[8]['fixedRows'][4]['values'][0]['value'])

        tmp_value2 = + s2fwithsep(AQSurvey[18]['fixedRows'][0]['values'][0]['value']) \
                     + s2fwithsep(AQSurvey[18]['fixedRows'][14]['values'][0]['value'])

        my_kpi['Stock Coverage DaysAIDA'] = round(try_div(tmp_value, tmp_value2) * 365, 2)
    else:
        my_kpi['Stock Coverage DaysAIDA'] = ariskparametri.NullValue


    my_kpi['EBIT'] = \
        my_kpi['EBITDA'] \
        - s2fwithsep(AQSurvey[18]['fixedRows'][12]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][13]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][15]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][16]['values'][0]['value'])

    my_kpi['EBITAIDA'] = s2fwithsep(AQSurvey[18]['fixedRows'][19]['values'][0]['value'])

    my_kpi['capitaleInvestitoNetto'] = \
        my_kpi['Net Working Capital'] \
        - s2fwithsep(AQSurvey[12]['fixedRows'][3]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[13]['fixedRows'][1]['values'][0]['value']) \
        + my_kpi['attivoFisso']

    if BILTYPE == '3':
        my_kpi['ROI'] = try_div(my_kpi['EBIT'], my_kpi['capitaleInvestitoNetto']) * 100

        tmp_value = + s2fwithsep(AQSurvey[18]['fixedRows'][19]['values'][0]['value'])

        tmp_value2 = + s2fwithsep(AQSurvey[11]['fixedRows'][9]['values'][0]['value']) \
                     + s2fwithsep(AQSurvey[14]['fixedRows'][0]['values'][0]['value']) \
                     + s2fwithsep(AQSurvey[14]['fixedRows'][4]['values'][0]['value']) \
                     + s2fwithsep(AQSurvey[14]['fixedRows'][6]['values'][0]['value']) \
                     + s2fwithsep(AQSurvey[14]['fixedRows'][8]['values'][0]['value'])

        my_kpi['ROIAIDA'] = round(try_div(tmp_value, tmp_value2) * 100, 2)

    else:
        my_kpi['ROI'] = ariskparametri.NullValue
        my_kpi['ROIAIDA'] = ariskparametri.NullValue

    my_kpi['EBITDA Margin'] = try_div(my_kpi['EBITDA'], my_kpi['Revenues']) * 100

    tmp_value = + my_kpi['Revenues'] + s2fwithsep(AQSurvey[17]['fixedRows'][5]['values'][0]['value'])
    my_kpi['EBITDA MarginAIDA'] = round(try_div(my_kpi['EBITDAAIDA'], tmp_value) * 100, 2)

    my_kpi['Interest On Sales'] = try_div(s2fwithsep(AQSurvey[19]['fixedRows'][2]['values'][0]['value']),
                                                   my_kpi['Revenues']) * 100
    tmp_value = s2fwithsep(AQSurvey[19]['fixedRows'][2]['values'][0]['value'])
    tmp_value2 = + my_kpi['Revenues'] + s2fwithsep(AQSurvey[17]['fixedRows'][5]['values'][0]['value'])
    my_kpi['Interest On SalesAIDA'] = round(try_div(tmp_value, tmp_value2) * 100, 2)


    if BILTYPE == '3':
        my_kpi['Net Financial Position'] = + s2fwithsep(AQSurvey[14]['fixedRows'][6]['values'][0]['value']) \
                                             - s2fwithsep(AQSurvey[8]['fixedRows'][32]['values'][0]['value'])

        my_kpi['Net Financial Position'] = my_kpi['Net Financial Position']

        my_kpi['Net Financial PositionAIDA'] = + s2fwithsep(AQSurvey[14]['fixedRows'][6]['values'][0]['value']) \
                                                  + s2fwithsep(AQSurvey[14]['fixedRows'][8]['values'][0]['value']) \
                                                  - s2fwithsep(AQSurvey[8]['fixedRows'][32]['values'][0]['value'])
    else:
        my_kpi['Net Financial Position'] = ariskparametri.NullValue

    if s2fwithsep(AQSurvey[25]['fixedRows'][3]['values'][0]['value']) != 0:
        my_kpi['Cash Flow'] = round(s2fwithsep(AQSurvey[25]['fixedRows'][3]['values'][0]['value']), 2)

    else:
        my_kpi['Cash Flow'] = ariskparametri.NullValue

    tmp_value = + s2fwithsep(AQSurvey[18]['fixedRows'][12]['values'][0]['value']) \
                + s2fwithsep(AQSurvey[18]['fixedRows'][13]['values'][0]['value'])

    my_kpi['Cash FlowAIDA'] = + s2fwithsep(AQSurvey[22]['fixedRows'][2]['values'][0]['value']) + tmp_value

    tmp_value = my_kpi['patrimonioNetto'] - s2fwithsep(AQSurvey[4]['fixedRows'][2]['values'][0]['value'])

    tmp_value2 = my_kpi['totaleDebiti'] + s2fwithsep(AQSurvey[15]['fixedRows'][2]['values'][0]['value'])

    my_kpi['patrimonioNettisutotaleDebitiNettoFondi'] = try_div(tmp_value, tmp_value2) * 100

    my_kpi['Quick Ratio Perc'] = my_kpi['Quick Ratio'] * 100

    my_kpi['attivoTotale'] = s2fwithsep(AQSurvey[9]['fixedRows'][3]['values'][0]['value'])

    if s2fwithsep(AQSurvey[25]['fixedRows'][3]['values'][0]['value']) != 0:
        my_kpi['Cash FlowsuAttivo'] = try_div(my_kpi['Cash Flow'], my_kpi['attivoTotale']) * 100

    else:
        my_kpi['Cash FlowsuAttivo'] = 0

    if BILTYPE == '3':
        my_kpi['indebitamentoPrevidsuAttivo'] = try_div(
            (+ s2fwithsep(AQSurvey[14]['fixedRows'][22]['values'][0]['value'])
             + s2fwithsep(AQSurvey[14]['fixedRows'][24]['values'][0]['value'])),
            my_kpi['attivoTotale']) * 100
    else:
        my_kpi['indebitamentoPrevidsuAttivo'] = 0

    if BILTYPE == '3':
        tmp_value = + s2fwithsep(AQSurvey[18]['fixedRows'][9]['values'][0]['value']) \
                    + s2fwithsep(AQSurvey[18]['fixedRows'][10]['values'][0]['value'])
        tmp_value2 = + s2fwithsep(AQSurvey[6]['fixedRows'][5]['values'][0]['value']) \
                     + s2fwithsep(AQSurvey[5]['fixedRows'][7]['values'][0]['value'])

        my_kpi['Amortisation Rate'] = round(try_div(tmp_value, tmp_value2) * 100, 2)

    else:
        my_kpi['Amortisation Rate'] = ariskparametri.NullValue

    my_kpi['Debt On Sales'] = round(try_div(my_kpi['totaleDebiti'], my_kpi['Revenues']) * 100, 2)

    my_kpi['Interest Coverage Ratio'] = s2fwithsep(AQSurvey[22]['fixedRows'][0]['values'][0]['value'])

    if s2fwithsep(AQSurvey[19]['fixedRows'][2]['values'][0]['value']) > 0:
        my_kpi['gradoInteressiPassivi'] = try_div((my_kpi['Interest Coverage Ratio']
                                                   + s2fwithsep(AQSurvey[19]['fixedRows'][2]['values'][0]['value'])),
                                                  s2fwithsep(AQSurvey[19]['fixedRows'][2]['values'][0]['value']))
    else:
        my_kpi['gradoInteressiPassivi'] = 0

    tmp_value = s2fwithsep(AQSurvey[19]['fixedRows'][2]['values'][0]['value'])
    my_kpi['gradoInteressiPassiviAIDA'] = round(try_div(my_kpi['EBITDAAIDA'], tmp_value), 2)

    if my_kpi['gradoInteressiPassiviAIDA'] <= 0 or my_kpi['gradoInteressiPassiviAIDA'] >= 500:
        my_kpi['gradoInteressiPassiviAIDA'] = 1000
    if my_kpi['gradoInteressiPassiviAIDA'] == 100000000 or my_kpi['gradoInteressiPassiviAIDA'] == 1000:
        if my_kpi['EBITDA MarginAIDA'] < 0:
            my_kpi['gradoInteressiPassiviAIDA'] = -1000
        else:
            my_kpi['gradoInteressiPassiviAIDA'] = 1000
    my_kpi['Shareholder Funds-Fixed Assets']

    # Risultato Operativo/Totale attività *100
    my_kpi['Return on Asset (ROA)'] = 100 * (s2fwithsep(AQSurvey[19]['fixedRows'][19]['values'][0]['value']) / my_kpi['attivoTotale'])
    
    # Risultato Operativo/(Ricavi+Altri ricavi) *100
    my_kpi['Return on Sales (ROS)'] = 100 * (s2fwithsep(AQSurvey[19]['fixedRows'][19]['values'][0]['value']) / (s2fwithsep(AQSurvey[18]['fixedRows'][5]['values'][0]['value'])+ my_kpi['Revenues']))
    
    # Utile netto/Patrimonio netto*100
    my_kpi['Return on Equity (ROE)'] = 100 * (s2fwithsep(AQSurvey[22]['fixedRows'][2]['values'][0]['value']) / my_kpi['patrimonioNetto'])
    
    # 1. abbreviato, 2. Micro, 3.Dettagliato
    my_kpi['Type of Financial Statement'] = 3

    if BILTYPE == '3':
        my_kpi['Altman'] = \
            try_div(0.717 * my_kpi['Net Working Capital'], my_kpi['attivoTotale']) + \
            try_div(0.847 * s2fwithsep(AQSurvey[24]['fixedRows'][0]['values'][0]['value']) * ratio,
                    my_kpi['attivoTotale']) + \
            try_div(3.107 * my_kpi['EBIT'], my_kpi['attivoTotale']) + \
            try_div(0.42 * my_kpi['patrimonioNetto'], my_kpi['totaleDebiti']) + \
            try_div(0.998 * my_kpi['Revenues'] * ratio, my_kpi['attivoTotale'])
    else:
        my_kpi['Altman'] = ariskparametri.NullValue

    return my_kpi


def computeKPIsBC(AQSurvey, ratio=1.0, change_revenue=1.0):
    BILTYPE = AQSurvey[2]['value']
    my_kpi = {}
    my_kpi['Revenues'] = s2fwithsep(AQSurvey[17]['fixedRows'][0]['values'][0]['value']) * change_revenue
    if BILTYPE == '3':
        my_kpi['Net Working Capital'] = 0
        my_kpi['Net Working Capital'] = \
            my_kpi['Net Working Capital'] \
            + s2fwithsep(AQSurvey[8]['fixedRows'][6]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][12]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][5]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[4]['fixedRows'][2]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][8]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][10]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][12]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][14]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][16]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][18]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[8]['fixedRows'][28]['values'][0]['value']) \
            + s2fwithsep(AQSurvey[9]['fixedRows'][2]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][0]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][2]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][4]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][8]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][10]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][14]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][16]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][18]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][20]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][22]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][24]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[14]['fixedRows'][26]['values'][0]['value']) \
            - s2fwithsep(AQSurvey[15]['fixedRows'][2]['values'][0]['value'])
    else:
        my_kpi['Net Working Capital'] = ariskparametri.NullValue

    my_kpi['EBITDA'] = \
        - s2fwithsep(AQSurvey[18]['fixedRows'][0]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][1]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][2]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][3]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][4]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][8]['values'][0]['value']) \
        + my_kpi['Revenues'] \
        + s2fwithsep(AQSurvey[17]['fixedRows'][3]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][14]['values'][0]['value'])

    my_kpi['EBITDAAIDA'] = \
        - s2fwithsep(AQSurvey[18]['fixedRows'][1]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][2]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][3]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][4]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][8]['values'][0]['value']) \
        + my_kpi['Revenues'] \
        + s2fwithsep(AQSurvey[17]['fixedRows'][5]['values'][0]['value']) \
        + s2fwithsep(AQSurvey[17]['fixedRows'][3]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][0]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][14]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][17]['values'][0]['value'])

    my_kpi['totaleRimanenze'] = s2fwithsep(AQSurvey[8]['fixedRows'][5]['values'][0]['value'])

    tmp_value = 0.0
    tmp_value = \
        tmp_value + s2fwithsep(AQSurvey[8]['fixedRows'][20]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[8]['fixedRows'][21]['values'][0]['value']) \
        + s2fwithsep(AQSurvey[8]['fixedRows'][32]['values'][0]['value']) \
        + s2fwithsep(AQSurvey[9]['fixedRows'][2]['values'][0]['value']) \
        + s2fwithsep(AQSurvey[8]['fixedRows'][5]['values'][0]['value'])

    tmp_value2 = 0.0
    tmp_value2 = \
        tmp_value2 \
        + s2fwithsep(AQSurvey[14]['fixedRows'][28]['values'][0]['value']) \
        + s2fwithsep(AQSurvey[15]['fixedRows'][2]['values'][0]['value'])

    my_kpi['Current Ratio'] = try_div(tmp_value, tmp_value2)

    my_kpi['Quick Ratio'] = try_div((tmp_value - my_kpi['totaleRimanenze']), tmp_value2)

    my_kpi['Net Short-Term Assets'] = tmp_value - tmp_value2

    my_kpi['totaleDebiti'] = s2fwithsep(AQSurvey[14]['fixedRows'][30]['values'][0]['value'])

    my_kpi['totaleDebitiEntre12Mesi'] = s2fwithsep(AQSurvey[14]['fixedRows'][28]['values'][0]['value'])

    tmp_value = s2fwithsep(AQSurvey[14]['fixedRows'][6]['values'][0]['value'])

    my_kpi['patrimonioNetto'] = s2fwithsep(AQSurvey[11]['fixedRows'][9]['values'][0]['value'])

    if BILTYPE == '3':
        my_kpi['levaFinanziaria'] = try_div(+tmp_value, my_kpi['patrimonioNetto'])
    else:
        my_kpi['levaFinanziaria'] = ariskparametri.NullValue

    my_kpi['gradoIndebitamento'] = try_div(my_kpi['totaleDebiti'], my_kpi['patrimonioNetto'])

    my_kpi['totaleDebitiOltre12Mesi'] = s2fwithsep(AQSurvey[14]['fixedRows'][29]['values'][0]['value'])

    my_kpi['Long-Term Liability Ratio'] = try_div(my_kpi['totaleDebitiOltre12Mesi'], my_kpi['patrimonioNetto'])

    my_kpi['attivoFisso'] = s2fwithsep(AQSurvey[7]['fixedRows'][11]['values'][0]['value'])

    my_kpi['Coverage Of Fixed Assets'] = try_div(my_kpi['patrimonioNetto'], my_kpi['attivoFisso'])

    my_kpi['Average Stock Turnover'] = try_div(my_kpi['totaleRimanenze'], (my_kpi['Revenues'] * ratio)) * 365

    if BILTYPE == '3':
        tmp_value = + s2fwithsep(AQSurvey[17]['fixedRows'][1]['values'][0]['value']) \
                    - s2fwithsep(AQSurvey[18]['fixedRows'][0]['values'][0]['value']) \
                    - s2fwithsep(AQSurvey[18]['fixedRows'][14]['values'][0]['value'])

        tmp_value = tmp_value * ratio
        my_kpi['Stock Coverage Days'] = try_div(my_kpi['totaleRimanenze'], -tmp_value) * 365
    else:
        my_kpi['Stock Coverage Days'] = ariskparametri.NullValue

    my_kpi['EBIT'] = \
        my_kpi['EBITDA'] \
        - s2fwithsep(AQSurvey[18]['fixedRows'][12]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][13]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][15]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[18]['fixedRows'][16]['values'][0]['value'])

    my_kpi['capitaleInvestitoNetto'] = \
        my_kpi['Net Working Capital'] \
        - s2fwithsep(AQSurvey[12]['fixedRows'][3]['values'][0]['value']) \
        - s2fwithsep(AQSurvey[13]['fixedRows'][1]['values'][0]['value']) \
        + my_kpi['attivoFisso']

    if BILTYPE == '3':
        my_kpi['ROI'] = try_div(my_kpi['EBIT'], my_kpi['capitaleInvestitoNetto']) * 100
    else:
        my_kpi['ROI'] = ariskparametri.NullValue

    my_kpi['EBITDA Margin'] = try_div(my_kpi['EBITDA'], my_kpi['Revenues']) * 100

    my_kpi['EBITDA MarginAIDA'] = try_div(my_kpi['EBITDAAIDA'], my_kpi['Revenues']) * 100

    my_kpi['Interest On Sales'] = try_div(s2fwithsep(AQSurvey[19]['fixedRows'][2]['values'][0]['value']),
                                                   my_kpi['Revenues']) * 100

    if BILTYPE == '3':
        my_kpi['Net Financial Position'] = + s2fwithsep(AQSurvey[14]['fixedRows'][6]['values'][0]['value']) \
                                             - s2fwithsep(AQSurvey[8]['fixedRows'][32]['values'][0]['value'])

        my_kpi['Net Financial Position'] = my_kpi['Net Financial Position']
    else:
        my_kpi['Net Financial Position'] = ariskparametri.NullValue

    if s2fwithsep(AQSurvey[25]['fixedRows'][3]['values'][0]['value']) != 0:
        my_kpi['Cash Flow'] = s2fwithsep(AQSurvey[25]['fixedRows'][3]['values'][0]['value'])

    else:
        my_kpi['Cash Flow'] = ariskparametri.NullValue

    tmp_value = my_kpi['patrimonioNetto'] - s2fwithsep(AQSurvey[4]['fixedRows'][2]['values'][0]['value'])

    tmp_value2 = my_kpi['totaleDebiti'] + s2fwithsep(AQSurvey[15]['fixedRows'][2]['values'][0]['value'])

    my_kpi['patrimonioNettisutotaleDebitiNettoFondi'] = try_div(tmp_value, tmp_value2) * 100

    my_kpi['indiceLiquiditaBreve'] = my_kpi['Quick Ratio'] * 100

    my_kpi['attivoTotale'] = s2fwithsep(AQSurvey[9]['fixedRows'][3]['values'][0]['value'])

    if s2fwithsep(AQSurvey[25]['fixedRows'][3]['values'][0]['value']) != 0:
        my_kpi['Cash FlowsuAttivo'] = try_div(my_kpi['Cash Flow'], my_kpi['attivoTotale']) * 100

    else:
        my_kpi['Cash FlowsuAttivo'] = 0

    if BILTYPE == '3':
        my_kpi['indebitamentoPrevidsuAttivo'] = try_div(
            (+ s2fwithsep(AQSurvey[14]['fixedRows'][22]['values'][0]['value'])
             + s2fwithsep(AQSurvey[14]['fixedRows'][24]['values'][0]['value'])),
            my_kpi['attivoTotale']) * 100
    else:
        my_kpi['indebitamentoPrevidsuAttivo'] = 0

    if BILTYPE == '3':

        tmp_value = + s2fwithsep(AQSurvey[18]['fixedRows'][9]['values'][0]['value']) \
                    + s2fwithsep(AQSurvey[18]['fixedRows'][10]['values'][0]['value'])

        tmp_value2 = + s2fwithsep(AQSurvey[6]['fixedRows'][5]['values'][0]['value']) \
                     + s2fwithsep(AQSurvey[5]['fixedRows'][7]['values'][0]['value'])

        my_kpi['Amortisation Rate'] = try_div(tmp_value, tmp_value2) * 100

    else:
        my_kpi['Amortisation Rate'] = ariskparametri.NullValue

    my_kpi['Debt On Sales'] = try_div(my_kpi['totaleDebiti'], my_kpi['Revenues']) * 100

    my_kpi['Interest Coverage Ratio'] = s2fwithsep(AQSurvey[22]['fixedRows'][0]['values'][0]['value'])

    if s2fwithsep(AQSurvey[19]['fixedRows'][2]['values'][0]['value']) > 0:
        my_kpi['gradoInteressiPassivi'] = try_div((my_kpi['Interest Coverage Ratio']
                                                   + s2fwithsep(AQSurvey[19]['fixedRows'][2]['values'][0]['value'])),
                                                  s2fwithsep(AQSurvey[19]['fixedRows'][2]['values'][0]['value']))
    else:
        my_kpi['gradoInteressiPassivi'] = 0

    if BILTYPE == '3':
        my_kpi['Altman'] = \
            try_div(0.717 * my_kpi['Net Working Capital'], my_kpi['attivoTotale']) + \
            try_div(0.847 * s2fwithsep(AQSurvey[24]['fixedRows'][0]['values'][0]['value']) * ratio,
                    my_kpi['attivoTotale']) + \
            try_div(3.107 * my_kpi['EBIT'], my_kpi['attivoTotale']) + \
            try_div(0.42 * my_kpi['patrimonioNetto'], my_kpi['totaleDebiti']) + \
            try_div(0.998 * my_kpi['Revenues'] * ratio, my_kpi['attivoTotale'])
    else:
        my_kpi['Altman'] = ariskparametri.NullValue

    return my_kpi


def generateOrgChartData(datiEconFin):
    
    stringLabel = ""

    orgChart = defaultdict(dict)
    stringLabel += 'Soci<div style="color:#000099;">'
    stringTooltip = ''

    for i in range(len(datiEconFin[3]['value'])):
        stringLabel = stringLabel \
                      + "<p>"+stringFromJson(datiEconFin[3]['value'][i]['COGNOMEPF'])\
                      + " " \
                      + stringFromJson(datiEconFin[3]['value'][i]['NOMEPF'])\
                      + " Quota " \
                      + datiEconFin[3]['value'][i]['QUOTE']+"%</p>"

    stringLabel = \
        stringLabel \
        + "<p>Numero di soci per la maggioranza: "\
        + stringFromJson(datiEconFin[4]['fixedRows'][0]['values'][0]['value'])\
        + "</p></div>"

    orgChart['soci']['label'] = stringLabel
    orgChart['soci']['tooltip'] = stringTooltip

    stringLabel = '<div style="color:#000099;">'
    stringTooltip = ''
    for i in range(len(datiEconFin[3]['value'])):
        if datiEconFin[3]['value'][i]['AMMN'] == 'si':
            stringLabel = stringLabel + "<p>" \
                          + stringFromJson(datiEconFin[3]['value'][i]['COGNOMEPF']) \
                          + " " + stringFromJson(datiEconFin[3]['value'][i]['NOMEPF'])
            if datiEconFin[3]['value'][i]['PERFISICA'] == 'si':
                stringLabel = stringLabel + " - Persona Fisica"
            else:
                stringLabel = stringLabel + ""
    for i in range(len(datiEconFin[3]['value'])):
        if datiEconFin[3]['value'][i]['AMMN'] == 'no':
            stringLabel = stringLabel + "<p>" + stringFromJson(datiEconFin[3]['value'][i]['COGNOMEPF']) \
                          + " " + stringFromJson(datiEconFin[3]['value'][i]['NOMEPF'])
            if datiEconFin[3]['value'][i]['PERFISICA'] == 'si':
                stringLabel = stringLabel + " - Persona Fisica"
            else:
                stringLabel = stringLabel + ""

    stringLabel = stringLabel + "</div>"

    orgChart['govAmm']['label'] = stringLabel
    orgChart['govAmm']['tooltip'] = stringTooltip

    stringLabel = '<div style="color:#000099;">'
    stringTooltip = ''
    for i in range(len(datiEconFin[8]['value'])):
        stringLabel = stringLabel + "<p>" + stringFromJson(datiEconFin[8]['value'][i]['COGNOME'])\
                      + " " + stringFromJson(datiEconFin[8]['value'][i]['NOME']) \
                      + " - " + stringFromJson(datiEconFin[8]['value'][i]['RUOLO'])
        if datiEconFin[8]['value'][i]['RISKINTERNO'] == 'si':
            stringLabel = stringLabel + " - Interno</p>"
        else:
            stringLabel = stringLabel + " - Esterno</p>"
    stringLabel = stringLabel + "</div>"

    orgChart['govSindaco']['label'] = stringLabel
    orgChart['govSindaco']['tooltip'] = stringTooltip

    stringLabel = 'Revisione<div style="color:#000099;">'
    stringTooltip = ''
    for i in range(len(datiEconFin[6]['value'])):
        if datiEconFin[6]['value'][i]['RUOLO'] != 'Sindaco unico':
            stringLabel = stringLabel + "<p>" + stringFromJson(datiEconFin[6]['value'][i]['COGNOMEPF']) + " - "\
                          + stringFromJson(datiEconFin[6]['value'][i]['RUOLO']) + " "
            if datiEconFin[6]['value'][i]['REVOBBLIGATORIA'] == 'si':
                stringLabel = stringLabel + " - Rev. Obbligatoria</p>"
            else:
                stringLabel = stringLabel + "</p>"

    stringLabel = stringLabel + "</div>"

    orgChart['govRev']['label'] = stringLabel
    orgChart['govRev']['tooltip'] = stringTooltip

    stringLabel = 'Keyman<div style="color:#000099;">'
    stringTooltip = ''
    for i in range(len(datiEconFin[8]['value'])):
        stringLabel = stringLabel \
                      + "<p>" + stringFromJson(datiEconFin[8]['value'][i]['COGNOME']) \
                      + " " + stringFromJson(datiEconFin[8]['value'][i]['NOME'])
        if datiEconFin[8]['value'][i]['RUOLO']:
            stringLabel = stringLabel + " - " + datiEconFin[8]['value'][i]['RUOLO']
        stringLabel = stringLabel + "</p>"

    stringLabel = stringLabel + "</div>"

    orgChart['govKeyman']['label'] = stringLabel
    orgChart['govKeyman']['tooltip'] = stringTooltip

    return orgChart


def check_risk_assessment(risk_data, anagrafic_data):
    risk_dict = dict()
    treatment_dict = dict()
    arr_severity = dict()
    post_trat_severity = dict()
    length_of_risks = 0

    risk_dict['period_start'] = datetime.strptime(risk_data["period_start"], '%Y-%m-%d %H:%M:%S').date()
    risk_dict['period_end'] = datetime.strptime(risk_data["period_end"], '%Y-%m-%d %H:%M:%S').date()
    for i in range(23, 32):
        arr = []
        position = []

        count = 0
        for idx in risk_data['input'][i]['value']:
            severity = \
                ariskparametri.probabilities[idx['RISKCOLUMPROB']] * \
                ariskparametri.impact_rate[idx['RISKCOLUMIMIMPACT']] * \
                ariskparametri.gravity[idx['RISKCOLUMSEV']]
            count += severity
            temp = ({'b': ariskparametri.probabilities[idx['RISKCOLUMPROB']],
                     'c': ariskparametri.impact_rate[idx['RISKCOLUMIMIMPACT']]})
            overlap = 0
            for elem in position:
                if temp == elem:
                    overlap = random.uniform(-0.1, 0.1)
            position.append(temp)
            try:
                arr.append({"implication": ariskparametri.implication[idx['RISKCOLUMIMPLI']],
                            "implication_code": idx['RISKCOLUMIMPLI'],
                            "severity": int(1 + (severity / 45 * 9)),
                            "severity_code": severity,
                            'risk_name': idx['RISKCOLUMTYPE'],
                            'probability': ariskparametri.probabilities[idx['RISKCOLUMPROB']] + overlap,
                            "probability_code": idx['RISKCOLUMPROB'],
                            "impact": ariskparametri.impact_rate[idx['RISKCOLUMIMIMPACT']] + overlap,
                            "impact_code": idx['RISKCOLUMIMIMPACT'],
                            "gravity": ariskparametri.gravity[idx['RISKCOLUMSEV']],

                            "gravity_code": idx['RISKCOLUMSEV'],
                            })

            except Exception as e:
                print(e)
                arr.append({"implication": "myerror",
                            "implication_code": "Non calcolabile",
                            "severity": 5,
                            "severity_code": -1,
                            'risk_name': "Non calcolabile",
                            'probability': ariskparametri.probabilities[idx['RISKCOLUMPROB']] + overlap,
                            "probability_code": idx['RISKCOLUMPROB'],
                            "impact": ariskparametri.impact_rate[idx['RISKCOLUMIMIMPACT']] + overlap,
                            "impact_code": idx['RISKCOLUMIMIMPACT'],
                            "gravity": ariskparametri.gravity[idx['RISKCOLUMSEV']],
                            "gravity_code": idx['RISKCOLUMSEV'],
                            })

        length_of_risks += len(arr)
        if count > 0:
            avg = count / (len(risk_data['input'][i]['value']) * 45)
        else:
            avg = 0
        arr_severity[risk_data['input'][i]['code']] = round(avg, 2)
        risk_dict[risk_data['input'][i]['code']] = arr
    

    arr_top_10 = OrderedDict()

    arr_cyber = []
    trattamento_cyber = risk_data['input'][23]['columns'][0]['options']

    count = 0
    for idx in risk_data['input'][23]['value']:
        for index in trattamento_cyber:
            if idx['RISKCOLUMTYPE'] == index:
                severity = \
                    ariskparametri.probabilities[idx['RISKCOLUMPROB']] * \
                    ariskparametri.impact_rate[idx['RISKCOLUMIMIMPACT']] * \
                    ariskparametri.gravity[idx['RISKCOLUMSEV']] * s2fwithsep(idx['RISKCOLUMMONIT'], 0) * 10
                count += severity
                arr_top_10[index] = severity

                if severity < 16:
                    severity_color = 'green'
                elif severity < 31:
                    severity_color = 'yellow'
                else:
                    severity_color = 'red'
                arr_cyber.append({'risk_name': index,
                                 'RISKCOLUMPRESIDIO': idx['RISKCOLUMPRESIDIO'],
                                 'RISKCOLUMPRESIDIOCOP': idx['RISKCOLUMPRESIDIOCOP'],
                                 'RISKCOLUMPRESIDIOALTRO': idx['RISKCOLUMPRESIDIOALTRO'],
                                 'RISKCOLUMPRESIDIOGIUR': idx['RISKCOLUMPRESIDIOGIUR'],
                                 'RISKCOLUMMONIT': idx['RISKCOLUMMONIT'],
                                 'Dati non trattati': 0,
                                 'Severity': int(1+(severity/45*9)),
                                 'Severity_color': severity_color,
                                 'annotation': ''
                                 })
                trattamento_cyber.remove(index)

    for risk in trattamento_cyber:
        arr_cyber.append({'risk_name': risk,
                         'RISKCOLUMPRESIDIO': 0,
                         'RISKCOLUMPRESIDIOCOP': 0,
                         'RISKCOLUMPRESIDIOALTRO': 0,
                         'RISKCOLUMPRESIDIOGIUR': 0,
                         'RISKCOLUMPRESIDIOASS': 0,
                         'RISKCOLUMPRESIDIOINT': 0,
                         'RISKCOLUMMONIT': 0,
                         'Dati non trattati': 100,
                         'Severity': 'nd',
                         'Severity_color': 'white',
                         'annotation': 'Informazione non fornita dall’utente'
                         })
    treatment_dict[risk_data['input'][23]['code']] = arr_cyber
    if count > 0:
        avg = count / (len(risk_data['input'][23]['value']) * 45)
    else:
        avg = 0

    arr_seismic = []
    trattamento_seismic = risk_data['input'][23]['columns'][0]['options']

    count = 0
    for idx in risk_data['input'][23]['value']:
        for index in trattamento_seismic:
            if idx['RISKCOLUMTYPE'] == index:
                severity = \
                    ariskparametri.probabilities[idx['RISKCOLUMPROB']] * \
                    ariskparametri.impact_rate[idx['RISKCOLUMIMIMPACT']] * \
                    ariskparametri.gravity[idx['RISKCOLUMSEV']] * s2fwithsep(idx['RISKCOLUMMONIT'], 0) * 10
                count += severity
                arr_top_10[index] = severity

                if severity < 16:
                    severity_color = 'green'
                elif severity < 31:
                    severity_color = 'yellow'
                else:
                    severity_color = 'red'
                arr_seismic.append({'risk_name': index,
                                 'RISKCOLUMPRESIDIO': idx['RISKCOLUMPRESIDIO'],
                                 'RISKCOLUMPRESIDIOCOP': idx['RISKCOLUMPRESIDIOCOP'],
                                 'RISKCOLUMPRESIDIOALTRO': idx['RISKCOLUMPRESIDIOALTRO'],
                                 'RISKCOLUMPRESIDIOGIUR': idx['RISKCOLUMPRESIDIOGIUR'],
                                 'RISKCOLUMMONIT': idx['RISKCOLUMMONIT'],
                                 'Dati non trattati': 0,
                                 'Severity': int(1+(severity/45*9)),
                                 'Severity_color': severity_color,
                                 'annotation': ''
                                 })
                trattamento_seismic.remove(index)

    for risk in trattamento_seismic:
        arr_seismic.append({'risk_name': risk,
                         'RISKCOLUMPRESIDIO': 0,
                         'RISKCOLUMPRESIDIOCOP': 0,
                         'RISKCOLUMPRESIDIOALTRO': 0,
                         'RISKCOLUMPRESIDIOGIUR': 0,
                         'RISKCOLUMPRESIDIOASS': 0,
                         'RISKCOLUMPRESIDIOINT': 0,
                         'RISKCOLUMMONIT': 0,
                         'Dati non trattati': 100,
                         'Severity': 'nd',
                         'Severity_color': 'white',
                         'annotation': 'Informazione non fornita dall’utente'
                         })
    treatment_dict[risk_data['input'][23]['code']] = arr_seismic
    if count > 0:
        avg = count / (len(risk_data['input'][23]['value']) * 45)
    else:
        avg = 0   
    
	post_trat_severity[risk_data['input'][31]['code']] = round(avg, 2)
    
    arr_top_10 = {k: v for k, v in sorted(arr_top_10.items(), key=lambda item: item[1], reverse=True)}
    arr_top_10 = list(arr_top_10.items())
    if len(arr_top_10) >= 10:
        pass
    else:
        length = len(arr_top_10)
        for _ in range(length, 11):
            arr_top_10.append((' ', 0))

    return risk_dict, treatment_dict, str(length_of_risks)+'/51', arr_top_10, arr_severity, post_trat_severity


def check_profilo_aziendale(json_data):
    json_dict = dict()
    if json_data:
        json_dict['period_start'] = datetime.strptime(json_data["period_start"], '%Y-%m-%d %H:%M:%S').date()
        json_dict['period_end'] = datetime.strptime(json_data["period_end"], '%Y-%m-%d %H:%M:%S').date()

    # print(json_dict)
    return json_dict


def check_indice_settoriali(kpi_data, ateco_code, last_index):
    ateco_arr = ateco_code.split('.')
    if int(ateco_arr[0]) <= 3:
        industry = 1
    elif int(ateco_arr[0]) <= 33 or (int(ateco_arr[0]) == 35 and (int(ateco_arr[1]) == 11 or int(ateco_arr[1]) == 21)):
        industry = 2
    elif 36 <= int(ateco_arr[0]) <= 39 \
            or (int(ateco_arr[0]) == 35 and (int(ateco_arr[1]) == 12 or
                                             int(ateco_arr[1]) == 13 or
                                             int(ateco_arr[1]) == 22 or
                                             int(ateco_arr[1]) == 30 or
                                             int(ateco_arr[1]) == 20)) \
            or (int(ateco_arr[0]) == 49 and int(ateco_arr[1]) == 50):
        industry = 3
    elif int(ateco_arr[0]) == 40:
        industry = 4
    elif 42 <= int(ateco_arr[0]) <= 43:
        industry = 5
    elif 45 <= int(ateco_arr[0]) <= 46:
        industry = 6
    elif int(ateco_arr[0]) == 47 or int(ateco_arr[0]) == 56:
        industry = 7
    elif int(ateco_arr[0]) == 55 or 49 <= int(ateco_arr[0]) <= 53:
        industry = 8
    elif 58 <= int(ateco_arr[0]) <= 63 or 69 <= int(ateco_arr[0]) <= 82:
        industry = 9
    elif 85 <= int(ateco_arr[0]) <= 96:
        industry = 10
    else:
        industry = 0
    threshold = {}
    if industry == 1:
        threshold['Interest On Sales'] = 2.8
        threshold['patrimonioNettisutotaleDebitiNettoFondi'] = 9.4
        threshold['indiceLiquiditaBreve'] = 92.1
        threshold['Cash FlowsuAttivo'] = 0.3
        threshold['indebitamentoPrevidsuAttivo'] = 5.6
    elif industry == 2:
        threshold['Interest On Sales'] = 3.0
        threshold['patrimonioNettisutotaleDebitiNettoFondi'] = 7.6
        threshold['indiceLiquiditaBreve'] = 93.7
        threshold['Cash FlowsuAttivo'] = 0.5
        threshold['indebitamentoPrevidsuAttivo'] = 4.9
    elif industry == 3:
        threshold['Interest On Sales'] = 2.6
        threshold['patrimonioNettisutotaleDebitiNettoFondi'] = 6.7
        threshold['indiceLiquiditaBreve'] = 84.2
        threshold['Cash FlowsuAttivo'] = 1.9
        threshold['indebitamentoPrevidsuAttivo'] = 6.5
    elif industry == 4:
        threshold['Interest On Sales'] = 3.8
        threshold['patrimonioNettisutotaleDebitiNettoFondi'] = 4.9
        threshold['indiceLiquiditaBreve'] = 108.0
        threshold['Cash FlowsuAttivo'] = 0.4
        threshold['indebitamentoPrevidsuAttivo'] = 3.8
    elif industry == 5:
        threshold['Interest On Sales'] = 2.8
        threshold['patrimonioNettisutotaleDebitiNettoFondi'] = 5.3
        threshold['indiceLiquiditaBreve'] = 101.1
        threshold['Cash FlowsuAttivo'] = 1.4
        threshold['indebitamentoPrevidsuAttivo'] = 5.3
    elif industry == 6:
        threshold['Interest On Sales'] = 2.1
        threshold['patrimonioNettisutotaleDebitiNettoFondi'] = 6.3
        threshold['indiceLiquiditaBreve'] = 101.4
        threshold['Cash FlowsuAttivo'] = 0.6
        threshold['indebitamentoPrevidsuAttivo'] = 2.9
    elif industry == 7:
        threshold['Interest On Sales'] = 1.5
        threshold['patrimonioNettisutotaleDebitiNettoFondi'] = 4.2
        threshold['indiceLiquiditaBreve'] = 89.8
        threshold['Cash FlowsuAttivo'] = 1.0
        threshold['indebitamentoPrevidsuAttivo'] = 7.8
    elif industry == 8:
        threshold['Interest On Sales'] = 1.5
        threshold['patrimonioNettisutotaleDebitiNettoFondi'] = 4.1
        threshold['indiceLiquiditaBreve'] = 86.0
        threshold['Cash FlowsuAttivo'] = 1.4
        threshold['indebitamentoPrevidsuAttivo'] = 10.2
    elif industry == 9:
        threshold['Interest On Sales'] = 1.8
        threshold['patrimonioNettisutotaleDebitiNettoFondi'] = 5.2
        threshold['indiceLiquiditaBreve'] = 95.4
        threshold['Cash FlowsuAttivo'] = 1.7
        threshold['indebitamentoPrevidsuAttivo'] = 11.9
    elif industry == 10:
        threshold['Interest On Sales'] = 2.7
        threshold['patrimonioNettisutotaleDebitiNettoFondi'] = 2.3
        threshold['indiceLiquiditaBreve'] = 69.8
        threshold['Cash FlowsuAttivo'] = 0.5
        threshold['indebitamentoPrevidsuAttivo'] = 14.6
    else:
        threshold['Interest On Sales'] = 0
        threshold['patrimonioNettisutotaleDebitiNettoFondi'] = 0
        threshold['indiceLiquiditaBreve'] = 0
        threshold['Cash FlowsuAttivo'] = 0
        threshold['indebitamentoPrevidsuAttivo'] = 0

    style_dict = dict()

    for i, elem in enumerate(kpi_data):
        arr = []
        check = False
        if elem['Interest On Sales'] >= threshold['Interest On Sales']:
            arr.append({'Interest On Sales': 'red'})
        else:
            arr.append({'Interest On Sales': 'green'})
            if i == last_index:
                check = True

        if elem['patrimonioNettisutotaleDebitiNettoFondi'] <= threshold['patrimonioNettisutotaleDebitiNettoFondi']:
            arr.append({'patrimonioNettisutotaleDebitiNettoFondi': 'red'})
        else:
            arr.append({'patrimonioNettisutotaleDebitiNettoFondi': 'green'})
            if i == last_index:
                check = True
        if elem['indiceLiquiditaBreve'] <= threshold['indiceLiquiditaBreve']:
            arr.append({'indiceLiquiditaBreve': 'red'})
        else:
            arr.append({'indiceLiquiditaBreve': 'green'})
            if i == last_index:
                check = True

        if elem['Cash FlowsuAttivo'] <= threshold['Cash FlowsuAttivo']:
            arr.append({'Cash FlowsuAttivo': 'red'})
        else:
            arr.append({'Cash FlowsuAttivo': 'green'})
            if i == last_index:
                check = True

        if elem['indebitamentoPrevidsuAttivo'] >= threshold['indebitamentoPrevidsuAttivo']:
            arr.append({'indebitamentoPrevidsuAttivo': 'red'})
        else:
            arr.append({'indebitamentoPrevidsuAttivo': 'green'})
            if i == last_index:
                check = True

        arr.append({'check': check})
        style_dict[i] = arr

    return style_dict


def perturb_revenue(aqsurvey1, change_revenue):
    aqsurvey = copy.deepcopy(aqsurvey1)
    
    change_debt_costs = s2fwithsep(aqsurvey[17]['fixedRows'][0]['values'][0]['value']) * change_revenue

    aqsurvey[18]['fixedRows'][0]['values'][0]['value'] = \
        str(s2fwithsep(aqsurvey[18]['fixedRows'][0]['values'][0]['value']) + change_debt_costs)
    
    aqsurvey[18]['fixedRows'][1]['values'][0]['value'] = \
        str(s2fwithsep(aqsurvey[18]['fixedRows'][1]['values'][0]['value']) + change_debt_costs)

    aqsurvey[18]['fixedRows'][4]['values'][0]['value'] = \
        str(s2fwithsep(aqsurvey[18]['fixedRows'][4]['values'][0]['value']) + change_debt_costs)

    return aqsurvey


def calculate_age(birth_date):
    today = date.today()
    age = today.year - birth_date - 1900
    return age


def average_age(lst):
    return sum(lst) / len(lst)


def compute_gov_kpi(jSonData, finance_data):
    gov_kpi = dict()
    print(finance_data[0])
    gov_kpi['GOV01'] = len(jSonData['InData']['EFQ']['input'][3]['value'])
    
    share = list()
    female_shareholder = list()
    for elem in jSonData['InData']['EFQ']['input'][3]['value']:
        share.append(int(elem['QUOTE']))
        if int(elem['RISKIVACF'][9:11]) <= 31:
            female_shareholder.append((int(elem['QUOTE']), 'MR'))
        elif int(elem['RISKIVACF'][9:11]) <= 71:
            female_shareholder.append((int(elem['QUOTE']), 'MRS'))
    
    count = 0
    add = 0
    for num in sorted(share):
        add += num
        count += 1
        if add >= 51:
            break
    gov_kpi['Number Of Stakeholder Individuals'] = count
    age_list = list()
    male = False
    female = False
    cf_manager = set()
    cf_shareholder = set()
    dictionary_manager = dict()
    list_manager = list()
    for i in jSonData['InData']['EFQ']['input'][5]['value']:
        age_list.append(calculate_age(int(i['RISKCF'][6:8])))
        if int(i['RISKCF'][9:11]) <= 31:
            male = True
        elif int(i['RISKCF'][9:11]) <= 71:
            female = True
        cf_manager.add(i['RISKCF'])
        if i['COGNOME'].lower()+i['NOME'].lower() not in list_manager:
            if i['COGNOME'] in dictionary_manager:
                dictionary_manager[i['COGNOME']] += 1
            else:
                dictionary_manager[i['COGNOME']] = 1
        list_manager.append(i['COGNOME'].lower()+i['NOME'].lower())

    if female and not male:
        gov_kpi['Shareholders Gender'] = 1
    elif male and not female:
        gov_kpi['Shareholders Gender'] = 2
    else:
        gov_kpi['Shareholders Gender'] = 3
    
	count = 0
    add = 0
    for tu in sorted(female_shareholder, key=lambda tup: (tup[0], tup[1]), reverse=True):
        add += tu[0]
        if tu[1] == 'MRS':
            count += 1
        if add > 50:
            break
    gov_kpi['Number Of Stakeholder Companies'] = count
    gov_kpi['Shareholder Share'] = len(jSonData['InData']['EFQ']['input'][5]['value'])
    sindaco = False
    presidente = False
    revisore = False
    company = False
    for elem in jSonData['InData']['EFQ']['input'][6]['value']:
        if elem['RUOLO'] == 'Sindaco unico':
            sindaco = True
        if elem['RUOLO'] == 'Presidente del collegio sindacale':
            presidente = True
        if elem['RUOLO'] == 'Revisore':
            revisore = True
        if elem['PERFISICA'] == 'Si':
            company = True
    if sindaco:
        gov_kpi['Members Of The Revision And Advisory Board'] = 1
    if presidente:
        gov_kpi['Members Of The Revision And Advisory Board'] = 2
    else:
        gov_kpi['Members Of The Revision And Advisory Board'] = 0
    if revisore and company:
        gov_kpi['Presence Of The Advisory Company '] = 1
    else:
        gov_kpi['Presence Of The Advisory Company '] = 0

    for j in jSonData['InData']['EFQ']['input'][6]['value']:
        if j['RUOLO'] != 'Revisore':
            age_list.append(calculate_age(int(j['RISKIVACF'][6:8])))
    gov_kpi['DM_eta'] = average_age(age_list)

    for tu in sorted(list_manager, key=lambda tup: (tup[0], tup[1]), reverse=True):
        add += tu[0]
        if tu[1] == 'MRS':
            count += 1
        if add > 50:
            break
    gov_kpi['Decision-Makers Gender'] = count
    if len(cf_manager) == 1:
        gov_kpi['DM Carica.1'] = 1
    elif len(cf_manager) == 2:
        gov_kpi['DM Carica.1'] = 2
    else:
        gov_kpi['DM Carica.1'] = 0
    for k in jSonData['InData']['EFQ']['input'][3]['value']:
        cf_shareholder.add(k['RISKIVACF'])
        if k['COGNOMEPF'].lower()+k['NOMEPF'].lower() not in list_manager:
            if k['COGNOMEPF'] in dictionary_manager:
                dictionary_manager[k['COGNOMEPF']] += 1
            else:
                dictionary_manager[k['COGNOMEPF']] = 1
            list_manager.append(k['COGNOMEPF'].lower()+k['NOMEPF'].lower())
    
	common_persons = cf_manager.intersection(cf_shareholder)
    gov_kpi['Decision-Makers Gender'] = len(common_persons)

    if len(dictionary_manager) == 0:
        gov_kpi['Number Of Decision-Makers By The Same Family'] = 0
    else:
        gov_kpi['Number Of Decision-Makers By The Same Family'] = max(dictionary_manager.values())
    share_family = dict()
    for item in jSonData['InData']['EFQ']['input'][3]['value']:
        if item['COGNOMEPF'] in share_family:
            share_family[item['COGNOMEPF']] += s2fwithsep(item['QUOTE'])
        else:
            share_family[item['COGNOMEPF']] = s2fwithsep(item['QUOTE'])
    if len(share_family) == 0:
        gov_kpi['Number Of Decision-Makers By The Same Family_3'] = 100
    else:
        gov_kpi['Number Of Decision-Makers By The Same Family_3'] = int(max(share_family.values()))

    revenue_last_year = s2fwithsep(jSonData['InData']['AQ'][0]['input'][17]['fixedRows'][0]['values'][0]['value'])
    last_year = datetime.strptime(jSonData['InData']['AQ'][0]['period_start'], '%Y-%m-%d %H:%M:%S').year
    list_year = list()
    for item in jSonData['InData']['AQLOCKED']:
        list_year.append(datetime.strptime(item["period_start"], '%Y-%m-%d %H:%M:%S').year)
    revenue = list()

    for year in sorted(list_year, reverse=True)[0:2]:
        for item in jSonData['InData']['AQLOCKED']:
            if datetime.strptime(item["period_start"], '%Y-%m-%d %H:%M:%S').year == year:
                revenue.append(s2fwithsep(item['input'][17]['fixedRows'][0]['values'][0]['value']))
    if len(revenue) == 2:
        trend_minus_one = revenue_last_year - revenue[0]
        trend_minus_two = revenue[0] - revenue[1]
    else:
        trend_minus_one = 0
        trend_minus_two = 0
    if trend_minus_one > 0 and trend_minus_two > 0:
        gov_kpi['Company Phase'] = 1
    elif trend_minus_one < 0 and trend_minus_two < 0:
        gov_kpi['Company Phase'] = -1
    else:
        gov_kpi['Company Phase'] = 0
    if revenue[1] != 0 and revenue[1]:
        gov_kpi['GOV24'] = int(round(((revenue_last_year - revenue[1]) / revenue[1]) * 100, 0))
    else:
        gov_kpi['GOV24'] = 0
    start_year = datetime.strptime(jSonData['InData']['BPA']['input'][2]['fixedRows'][0]['values'][3]['value'],
                                   '%d/%m/%Y').year
    gov_kpi['GOV22'] = last_year - start_year
    gov_kpi['GOV27'] = finance_data[0][0]
    gov_kpi['GOV28'] = revenue_last_year
    gov_kpi['ATT12'] = finance_data[0][1]

    
    ind_gov = []
    for ind in ariskparametri.gov_indices:
        ind_gov = np.append(ind_gov, gov_kpi[ind])
    gov_data = pd.DataFrame(columns=ariskparametri.gov_indices)
    gov_data.loc[0] = ind_gov
    return gov_data.copy(), gov_kpi.copy()
