from __future__ import print_function

from flask import Flask, render_template, request, jsonify
import pandas as pd

# import json
from json2html import *

import numpy as np
import os

import ariskanalisi as arisk
from datetime import datetime, timedelta
from jinja2 import Environment, FileSystemLoader, Template

# from typing import Any
import copy
import random
from ariskutil import myprint, perturbAQ
from datetime import date

# from dateutil.relativedelta import *
from babel.numbers import format_currency

import ariskparametri
import warnings
warnings.filterwarnings("ignore")

app = Flask(__name__,
            static_folder='static',
            template_folder='templates')

@app.template_filter()
def mynewcurrency(value):
    # return format_currency(value*1000, '', locale='it_IT', currency_digits=False)
    return format_currency(value, 'EUR', locale='it_IT', currency_digits=False)


app.jinja_env.filters['mynewcurrency'] = mynewcurrency


@app.route("/hello")
def hello():
    myOutput = "ARisk Engine V1.0.12"
    return myOutput

@app.route("/RFRBAI", methods=['POST'])

def RFRBAI():
    global myCompanyData
    jSonData = request.get_json()
    os.chdir(ariskparametri.ariskBaseDir)
    if len(jSonData['InData']) == 0 or jSonData['InData']['AQ'] is None:
        env = Environment(loader=FileSystemLoader(os.path.join(os.path.dirname(__file__), 'template-report')))
        template = env.get_template('templateSelectIncomplete.html')
        myOutput = template.render()
        return myOutput

    if not arisk.checkSurveyData1(jSonData):
        env = Environment(loader=FileSystemLoader(os.path.join(os.path.dirname(__file__), 'template-report')))
        template = env.get_template('templateIncomplete.html')
        myOutput = template.render(Azienda=jSonData['InData']['ANA']['name'])
        return myOutput
    else:
        myAnagrData = [(jSonData['InData']['ANA']['name'],
                        jSonData['InData']['ANA']['piva'],
                        jSonData['InData']['ANA']['ateco_2007_code'],
                        datetime.strptime(jSonData['InData']['EFQ']['period_start'], '%Y-%m-%d %H:%M:%S').date(),
                        datetime.strptime(jSonData['InData']['EFQ']['period_end'], '%Y-%m-%d %H:%M:%S').date(),

                        datetime.strptime("2020-04-01 00:00:00", '%Y-%m-%d %H:%M:%S').date(),
                        datetime.strptime("2020-04-01 00:00:00", '%Y-%m-%d %H:%M:%S').date(),)
                       ]

        indFamiliarita = [jSonData['InData']['EFQ']['input'][3],
                          jSonData['InData']['EFQ']['input'][4],
                          jSonData['InData']['EFQ']['input'][5],
                          jSonData['InData']['EFQ']['input'][6]
                          ]
        # check the survey type
        if 'INDFINANZIARI' in jSonData:
            indFinanziari = jSonData['InData']['AQLOCKED']['input'][2]['fixedRows']
            myFinData = [(float(indFinanziari[8]['values'][0]['value']),
                          float(indFinanziari[9]['values'][0]['value']),
                          float(indFinanziari[10]['values'][0]['value']),
                          float(indFinanziari[5]['values'][0]['value']),
                          float(indFinanziari[11]['values'][0]['value']),
                          float(indFinanziari[12]['values'][0]['value']),
                          float(indFinanziari[13]['values'][0]['value']),
                          float(indFinanziari[14]['values'][0]['value']),
                          float(indFinanziari[15]['values'][0]['value']),
                          float(indFinanziari[16]['values'][0]['value']),
                          float(indFinanziari[17]['values'][0]['value']),
                          float(indFinanziari[18]['values'][0]['value']),
                          float(indFinanziari[19]['values'][0]['value']),
                          float(indFinanziari[20]['values'][0]['value']),
                          float(indFinanziari[21]['values'][0]['value']))
                         ]

            myAnagrafica = pd.DataFrame.from_records(myAnagrData, columns=ariskparametri.anagraficaCols_RFRBAI)
            myCompanyData = pd.DataFrame.from_records(myFinData, columns=ariskparametri.x_data_cols)
            risk_assessment, risk_treatment, length_of_risks, top_10, sev_array = \
                arisk.check_risk_assessment(jSonData['InData']['EFQ'], jSonData['InData']['ANA'])
            myOutput = arisk.analyze_company(myAnagrafica, myCompanyData, indFamiliarita, sev_array)
            
        else:
            
            myAnagrData = [
                'Depositato', 'D',
                jSonData['InData']['ANA']['name'],
                jSonData['InData']['ANA']['piva'],
                jSonData['InData']['ANA']['ateco_2007_code'],
                datetime.strptime(jSonData['InData']['EFQ']['period_start'], '%Y-%m-%d %H:%M:%S').date(),
                datetime.strptime(jSonData['InData']['EFQ']['period_end'], '%Y-%m-%d %H:%M:%S').date(),
                datetime.strptime("2020-04-01 00:00:00", '%Y-%m-%d %H:%M:%S').date(),
                datetime.strptime("2020-04-01 00:00:00", '%Y-%m-%d %H:%M:%S').date()
                ]

            datiEconFin = jSonData['InData']['EFQ']['input']
            random.seed(int(jSonData['InData']['ANA']['piva']))

            tmpRes = 0.0
            tmpParamValues = list()
            tmpParamNames = list()

            resFR = list()
            resParamValues = list()
            resParamNames = list()
            resKPIs = list()
            resAnagrafica = list()
            resParamIdx = list()
            orderDepositati = []
            if jSonData['InData']['AQLOCKED']:
                last_index = len(jSonData['InData']['AQLOCKED']) - 1
            else:
                last_index = len(jSonData['InData']['AQ']) - 1

            resBC = list()
            resRISK = list()
            datiEconFin = jSonData['InData']['EFQ']['input']
            orgChart = arisk.generateOrgChartData(datiEconFin)
            risk_assessment, risk_treatment, length_of_risks, top_10, sev_array, post_trat_array = \
                arisk.check_risk_assessment(jSonData['InData']['EFQ'], jSonData['InData']['ANA'])

            if jSonData['InData']['AQLOCKED'] is not None:
                for i in range(len(jSonData['InData']['AQLOCKED'])):
                    orderDepositati.append(
                        datetime.strptime(jSonData['InData']['AQLOCKED'][i]['period_end'], '%Y-%m-%d %H:%M:%S').date())
                orderDepositatiArray = np.array(orderDepositati)
                res = np.argsort(orderDepositatiArray)

                for index, i in enumerate(res, start=1):
                    indFinanziari2 = jSonData['InData']['AQLOCKED'][i]['input']
                    myAnagrData[0] = 'Bilancio annuale depositato'
                    myAnagrData[1] = 'D{0}'.format(index)
                    tmpDate = myAnagrData[8]
                    myAnagrData[7] = datetime.strptime(jSonData['InData']['AQLOCKED'][i]['period_start'],
                                                       '%Y-%m-%d %H:%M:%S').date()
                    myAnagrData[8] = datetime.strptime(jSonData['InData']['AQLOCKED'][i]['period_end'],
                                                       '%Y-%m-%d %H:%M:%S').date()
                    resAnagrafica.append(copy.deepcopy(myAnagrData))
                    KPIs = arisk.computeKPIs(indFinanziari2)
                    resKPIs.append(copy.deepcopy(KPIs))
                    indCrisi = []
                    for x in ariskparametri.finMLColumns:
                        indCrisi = np.append(indCrisi, KPIs[x])
                    myCompanyData = pd.DataFrame(columns=ariskparametri.x_data_cols)
                    myCompanyData.loc[0] = indCrisi
                    tmpRes, tmpParamNames, tmpParamValues, tmpParamIdx = arisk.analyze_company_rf(myCompanyData)
                    tmpBC = arisk.analyze_companyBC(myCompanyData, post_trat_array)
                    resBC.append(copy.deepcopy(tmpBC))
                    resFR.append(copy.deepcopy(tmpRes))
                    resParamValues.append(copy.deepcopy(tmpParamValues))
                    resParamNames.append(copy.deepcopy(tmpParamNames))
                    resParamIdx.append(copy.deepcopy(tmpParamIdx))

            orderAperti = []
            for i in range(len(jSonData['InData']['AQ'])):
                orderAperti.append(
                    datetime.strptime(jSonData['InData']['AQ'][i]['period_end'], '%Y-%m-%d %H:%M:%S').date())
            orderApertiArray = np.array(orderAperti)
            res = np.argsort(orderApertiArray)

            for index, i in enumerate(res, start=1):
                indFinanziari2 = jSonData['InData']['AQ'][i]['input']
                myAnagrData[0] = 'Bilancio di periodo'
                myAnagrData[1] = 'I{0}'.format(index)
                tmpDate = myAnagrData[8]
                myAnagrData[7] = datetime.strptime(jSonData['InData']['AQ'][i]['period_start'],
                                                   '%Y-%m-%d %H:%M:%S').date()
                myAnagrData[8] = datetime.strptime(jSonData['InData']['AQ'][i]['period_end'],
                                                   '%Y-%m-%d %H:%M:%S').date()

                delta = (myAnagrData[8] - myAnagrData[7]).days
                if delta > 0:
                    indexRatio = delta / 365.0
                else:
                    indexRatio = 1

                resAnagrafica.append(copy.deepcopy(myAnagrData))
                KPIs = arisk.computeKPIs(indFinanziari2, indexRatio)
                resKPIs.append(copy.deepcopy(KPIs))
                indCrisi = []
                for x in ariskparametri.finMLColumns:
                    indCrisi = np.append(indCrisi, KPIs[x])
                myCompanyData = pd.DataFrame(columns=ariskparametri.x_data_cols)
                myCompanyData.loc[0] = indCrisi
                tmpRes, tmpParamNames, tmpParamValues, tmpParamIdx = \
                    arisk.analyze_company_rf(myCompanyData, indexRatio)
                tmpBC = arisk.analyze_companyBC(myCompanyData, post_trat_array)
                resBC.append(copy.deepcopy(tmpBC))
                for change in [0.0, 0.1, 0.2, 0.3, 0.4, 0.5]:
                    perturbed_json_values = arisk.perturb_revenue(indFinanziari2, change)
                    KPIs = arisk.computeKPIs(perturbed_json_values, indexRatio)
                    indCrisi = []
                    for x in ariskparametri.finMLColumns:
                        indCrisi = np.append(indCrisi, KPIs[x])
                    myCompanyData = pd.DataFrame(columns=ariskparametri.x_data_cols)
                    myCompanyData.loc[0] = indCrisi
                    tmpBC = arisk.analyze_companyBC(myCompanyData, post_trat_array)
                resFR.append(copy.deepcopy(tmpRes))
                resParamValues.append(copy.deepcopy(tmpParamValues))
                resParamNames.append(copy.deepcopy(tmpParamNames))
                resParamIdx.append(copy.deepcopy(tmpParamIdx))

            gov_data, gov_kpi = arisk.compute_gov_kpi(jSonData['InData']['EFQ'],
                                                      np.array(myCompanyData[['ATT11', 'ATT12']]))
            resRISK = arisk.analyze_companyGov(gov_data)

            profilo_aziendale = arisk.check_profilo_aziendale(jSonData['InData']['BPA'])
            if jSonData['InData']['ANA']['ateco_2007_code'] and resKPIs:
                indice_settoriali = arisk.check_indice_settoriali(resKPIs, jSonData['InData']['ANA']['ateco_2007_code'],
                                                                  last_index)
            else:
                indice_settoriali = 'ND'

            myprint(resBC)
            myprint(resAnagrafica)
            myprint(resParamValues)
            myprint(resParamNames)
            myprint(resParamIdx)

            if len(resAnagrafica) == 0:
                resAnagrafica[0] = 'Dato non fornito'
                resAnagrafica[1] = ' '
                resAnagrafica[2] = 'Nessuna'
                resAnagrafica[7] = ' '
                resAnagrafica[8] = ' '

            env = Environment(loader=FileSystemLoader(os.path.join(os.path.dirname(__file__), 'template-report')))
            env.filters['mynewcurrency'] = mynewcurrency
            template = env.get_template('new-template-long2.html')
            myOutput = template.render(resAnagrafica=resAnagrafica,
                                       indice_settoriali=indice_settoriali,
                                       last_index=last_index,
                                       length_of_risks=length_of_risks,
                                       profilo_aziendale=profilo_aziendale,
                                       resParamValues=resParamValues,
                                       resParamNames=resParamNames,
                                       resKPIs=resKPIs,
                                       resFR=resFR,
                                       resBC=resBC,
                                       resRISK=resRISK,
                                       reportCreated=date.today(),
                                       orgChartData=orgChart,
                                       risk_assessment=risk_assessment,
                                       risk_treatment=risk_treatment,
                                       top_10=top_10
                                       )

    return myOutput


@app.route("/RFRCAI", methods=['POST'])
def RFRCAI():
    jSonData = request.get_json()

    if jSonData["oType"] == 0:
        result = json2html.convert(json=jSonData)
        result = jsonify(jSonData)
    else:
        result = jsonify(jSonData)

    return result


@app.route("/Temp", methods=['GET'])
def temp():
    env = Environment(loader=FileSystemLoader(os.path.join(os.path.dirname(__file__), 'template-report')))
    template = env.get_template('new-template-long2.html')
    myOutput = template.render()

    return myOutput


if __name__ == "__main__":
    app.run(debug=True)
